@@ -224,13 +224,15 @@ class AbstractSamGeneration(RevPySam, ScheduledLossesMixin, ABC):
         site_sys_inputs : dict
             Optional set of site-specific SAM system inputs to complement the
             site-agnostic inputs.
-        meta : pd.DataFrame
-            1D table with resource meta data.
+        meta : pd.DataFrame | pd.Series
+            Meta data corresponding to the resource input for the single
+            location. Should include values for latitude, longitude, elevation,
+            and timezone.
 
         Returns
         -------
-        meta : pd.DataFrame
-            1D table with resource meta data. Will include "timezone"
+        meta : pd.DataFrame | pd.Series
+            Datafram or series for a single site. Will include "timezone"
             and "elevation" from the sam and site system inputs if found.
         """
 
@@ -429,12 +431,13 @@ class AbstractSamGeneration(RevPySam, ScheduledLossesMixin, ABC):
         bias_correct : None | pd.DataFrame
             None if not provided or extracted DataFrame with wind or solar
             resource bias correction table. This has columns: gid (can be index
-            name), adder, scalar. If both adder and scalar are present, the
-            wind or solar resource is corrected by (res*scalar)+adder. If
-            either adder or scalar is not present, scalar defaults to 1 and
-            adder to 0. Only windspeed or GHI+DNI are corrected depending on
-            the technology. GHI and DNI are corrected with the same correction
-            factors.
+            name), adder, scalar. The gid field should match the true resource
+            gid regardless of the optional gid_map input. If both adder and
+            scalar are present, the wind or solar resource is corrected by
+            (res*scalar)+adder. If either adder or scalar is not present,
+            scalar defaults to 1 and adder to 0. Only windspeed or GHI+DNI are
+            corrected depending on the technology. GHI and DNI are corrected
+            with the same correction factors.
 
         Returns
         -------
@@ -530,7 +533,7 @@ class AbstractSamGenerationFromWeatherFile(AbstractSamGeneration, ABC):
             required variables to run the respective SAM simulation.
             Remapping will be done to convert typical NSRDB/WTK names
             into SAM names (e.g. DNI -> dn and wind_speed -> windspeed).
-        meta : pd.DataFrame | pd.Series
+        meta : pd.Series
             Meta data corresponding to the resource input for the single
             location. Should include values for latitude, longitude,
             elevation, and timezone.
@@ -550,7 +553,7 @@ class AbstractSamGenerationFromWeatherFile(AbstractSamGeneration, ABC):
             required variables to run the respective SAM simulation.
             Remapping will be done to convert typical NSRDB/WTK names
             into SAM names (e.g. DNI -> dn and wind_speed -> windspeed).
-        meta : pd.DataFrame | pd.Series
+        meta : pd.Series
             Meta data corresponding to the resource input for the single
             location. Should include values for latitude, longitude,
             elevation, and timezone.
@@ -678,7 +681,7 @@ class AbstractSamSolar(AbstractSamGeneration, ABC):
             variables to run the respective SAM simulation. Remapping will be
             done to convert typical NSRDB/WTK names into SAM names (e.g. DNI ->
             dn and wind_speed -> windspeed)
-        meta : pd.DataFrame | pd.Series
+        meta : pd.Series
             Meta data corresponding to the resource input for the single
             location. Should include values for latitude, longitude, elevation,
             and timezone.
@@ -718,9 +721,9 @@ class AbstractSamSolar(AbstractSamGeneration, ABC):
             if var != 'time_index':
 
                 # ensure that resource array length is multiple of 8760
-                arr = np.roll(
-                    self.ensure_res_len(arr, time_index),
-                    int(self._meta['timezone'] * self.time_interval))
+                arr = self.ensure_res_len(arr, time_index)
+                n_roll = int(self._meta['timezone'] * self.time_interval)
+                arr = np.roll(arr, n_roll)
 
                 if var in irrad_vars:
                     if np.min(arr) < 0:
@@ -766,6 +769,55 @@ class AbstractSamPv(AbstractSamSolar, ABC):
                  output_request=None, drop_leap=False):
         """Initialize a SAM solar object.
 
+        See the PySAM :py:class:`~PySAM.Pvwattsv8.Pvwattsv8` (or older
+        version model) documentation for the configuration keys required
+        in the `sam_sys_inputs` config. You may also include the
+        following ``reV``-specific keys:
+
+            - ``reV_outages`` : Specification for ``reV``-scheduled
+              stochastic outage losses. For example::
+
+                    outage_info = [
+                        {
+                            'count': 6,
+                            'duration': 24,
+                            'percentage_of_capacity_lost': 100,
+                            'allowed_months': ['January', 'March'],
+                            'allow_outage_overlap': True
+                        },
+                        {
+                            'count': 10,
+                            'duration': 1,
+                            'percentage_of_capacity_lost': 10,
+                            'allowed_months': ['January'],
+                            'allow_outage_overlap': False
+                        },
+                        ...
+                    ]
+
+              See the description of
+              :meth:`~reV.losses.scheduled.ScheduledLossesMixin.add_scheduled_losses`
+              or the
+              `reV losses demo notebook <https://tinyurl.com/4d7uutt3/>`_
+              for detailed instructions on how to specify this input.
+            - ``reV_outages_seed`` : Integer value used to seed the RNG
+              used to compute stochastic outage losses.
+            - ``time_index_step`` : Integer representing the step size
+              used to sample the ``time_index`` in the resource data.
+              This can be used to reduce temporal resolution (i.e. for
+              30 minute NSRDB input data, ``time_index_step=1`` yields
+              the full 30 minute time series as output, while
+              ``time_index_step=2`` yields hourly output, and so forth).
+
+              .. Note:: The reduced data shape (i.e. after applying a
+                        step size of `time_index_step`) must still be an
+                        integer multiple of 8760, or the execution will
+                        fail.
+
+            - ``clearsky`` : Boolean flag value indicating wether
+              computation should use clearsky resource data to compute
+              generation data.
+
         Parameters
         ----------
         resource : pd.DataFrame
@@ -793,6 +845,7 @@ class AbstractSamPv(AbstractSamSolar, ABC):
         """
 
         # need to check tilt=lat and azimuth for pv systems
+        meta = self._parse_meta(meta)
         sam_sys_inputs = self.set_latitude_tilt_az(sam_sys_inputs, meta)
 
         super().__init__(resource, meta, sam_sys_inputs,
@@ -800,6 +853,39 @@ class AbstractSamPv(AbstractSamSolar, ABC):
                          output_request=output_request,
                          drop_leap=drop_leap)
 
+    def set_resource_data(self, resource, meta):
+        """Set NSRDB resource data arrays.
+
+        Parameters
+        ----------
+        resource : pd.DataFrame
+            Timeseries solar or wind resource data for a single location with a
+            pandas DatetimeIndex.  There must be columns for all the required
+            variables to run the respective SAM simulation. Remapping will be
+            done to convert typical NSRDB/WTK names into SAM names (e.g. DNI ->
+            dn and wind_speed -> windspeed)
+        meta : pd.Series
+            Meta data corresponding to the resource input for the single
+            location. Should include values for latitude, longitude, elevation,
+            and timezone.
+
+        Raises
+        ------
+        ValueError : If lat/lon outside of -90 to 90 and -180 to 180,
+                     respectively.
+
+        """
+        bad_location_input = ((meta['latitude'] < -90)
+                              | (meta['latitude'] > 90)
+                              | (meta['longitude'] < -180)
+                              | (meta['longitude'] > 180))
+        if bad_location_input.any():
+            raise ValueError("Detected latitude/longitude values outside of "
+                             "the range -90 to 90 and -180 to 180, "
+                             "respectively. Please ensure input resource data"
+                             "locations conform to these ranges. ")
+        return super().set_resource_data(resource, meta)
+
     @staticmethod
     def set_latitude_tilt_az(sam_sys_inputs, meta):
         """Check if tilt is specified as latitude and set tilt=lat, az=180 or 0
@@ -808,8 +894,10 @@ class AbstractSamPv(AbstractSamSolar, ABC):
         ----------
         sam_sys_inputs : dict
             Site-agnostic SAM system model inputs arguments.
-        meta : pd.DataFrame
-            1D table with resource meta data.
+        meta : pd.Series
+            Meta data corresponding to the resource input for the single
+            location. Should include values for latitude, longitude, elevation,
+            and timezone.
 
         Returns
         -------
@@ -848,6 +936,19 @@ class AbstractSamPv(AbstractSamSolar, ABC):
                                  sam_sys_inputs['azimuth']))
         return sam_sys_inputs
 
+    def system_capacity_ac(self):
+        """Get AC system capacity from SAM inputs.
+
+        NOTE: AC nameplate = DC nameplate / ILR
+
+        Returns
+        -------
+        cf_profile : float
+            AC nameplate = DC nameplate / ILR
+        """
+        return (self.sam_sys_inputs['system_capacity']
+                / self.sam_sys_inputs['dc_ac_ratio'])
+
     def cf_mean(self):
         """Get mean capacity factor (fractional) from SAM.
 
@@ -861,6 +962,19 @@ class AbstractSamPv(AbstractSamSolar, ABC):
         """
         return self['capacity_factor'] / 100
 
+    def cf_mean_ac(self):
+        """Get mean AC capacity factor (fractional) from SAM.
+
+        NOTE: This value only available in PVWattsV8 and up.
+
+        Returns
+        -------
+        output : float
+            Mean AC capacity factor (fractional).
+            PV AC CF is calculated as AC power / AC nameplate.
+        """
+        return self['capacity_factor_ac'] / 100
+
     def cf_profile(self):
         """Get hourly capacity factor (frac) profile in local timezone.
         See self.outputs attribute for collected output data in UTC.
@@ -876,6 +990,22 @@ class AbstractSamPv(AbstractSamSolar, ABC):
         """
         return self.gen_profile() / self.sam_sys_inputs['system_capacity']
 
+    def cf_profile_ac(self):
+        """Get hourly AC capacity factor (frac) profile in local timezone.
+        See self.outputs attribute for collected output data in UTC.
+
+        NOTE: PV AC capacity factor is the AC power production / the AC
+        nameplate. AC nameplate = DC nameplate / ILR
+
+        Returns
+        -------
+        cf_profile : np.ndarray
+            1D numpy array of capacity factor profile.
+            Datatype is float32 and array length is 8760*time_interval.
+            PV AC CF is calculated as AC power / AC nameplate.
+        """
+        return self.gen_profile() / self.system_capacity_ac()
+
     def gen_profile(self):
         """Get AC inverter power generation profile (local timezone) in kW.
         This is an alias of the "ac" SAM output variable if PySAM version>=3.
@@ -949,13 +1079,16 @@ class AbstractSamPv(AbstractSamSolar, ABC):
 
         if output_lookup is None:
             output_lookup = {'cf_mean': self.cf_mean,
+                             'cf_mean_ac': self.cf_mean_ac,
                              'cf_profile': self.cf_profile,
+                             'cf_profile_ac': self.cf_profile_ac,
                              'annual_energy': self.annual_energy,
                              'energy_yield': self.energy_yield,
                              'gen_profile': self.gen_profile,
                              'ac': self.ac,
                              'dc': self.dc,
-                             'clipped_power': self.clipped_power
+                             'clipped_power': self.clipped_power,
+                             'system_capacity_ac': self.system_capacity_ac,
                              }
 
         super().collect_outputs(output_lookup=output_lookup)
@@ -1175,20 +1308,9 @@ class TroughPhysicalHeat(AbstractSamGenerationFromWeatherFile):
 class Geothermal(AbstractSamGenerationFromWeatherFile):
     """reV-SAM geothermal generation.
 
-    Unlike wind or solar, reV geothermal dynamically sets the size of a
-    geothermal plant. In particular, the nameplate capacity is set to
-    match the resource potential (obtained form the input data) for each
-    site. As a result, reV allows users to input ``capital_cost_per_kw``
-    and ``fixed_operating_cost_per_kw`` instead of the flat
-    ``capital_cost`` and ``fixed_operating_cost`` values, respectively,
-    in the SAM technology config. If these inputs are detected, reV
-    calculates the total ``capital_cost`` and ``fixed_operating_cost``
-    based on the plant size and automatically adds them to the SAM
-    config on a per-site basis.
-
-    As of 12/20/2022, the resource potential input is only used to
-    calculate the number of well replacements during the lifetime of a
-    geothermal plant. It was decided that reV would not model well
+    As of 12/20/2022, the resource potential input in SAM is only used
+    to calculate the number of well replacements during the lifetime of
+    a geothermal plant. It was decided that reV would not model well
     replacements. Therefore, reV sets the resource potential to match
     (or be just above) the gross potential so that SAM does not throw
     any errors.
@@ -1198,12 +1320,156 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
     reV currently generates an empty weather file to pass to SAM. This
     behavior can be easily updated in the future should the SAM GETEM
     module start using weather data.
+
+    See the PySAM :py:class:`~PySAM.Geothermal.Geothermal` documentation
+    for the configuration keys required in the `sam_sys_inputs` config.
+    Some notable keys include (non-exhaustive):
+
+        - ``resource_type`` : Integer flag representing either
+          Hydrothermal (0) or EGS (1) resource. Only values of 0 or 1
+          allowed.
+        - ``resource_potential`` : Total resource potential at location
+          (in MW).
+
+          .. Important:: ``reV`` automatically sets the resource
+             potential to match the gross potential (see documentation
+             above), so this key should be left out of the config (it
+             will be overridden in any case).
+
+        - ``resource_temp`` : Temperature of resource (in C).
+
+          .. Important:: This value is set by ``reV`` based on the
+             user's geothermal resource data input. To override this
+             behavior, users *may* specify their own ``resource_temp``
+             value (either a single value for all sites in the SAM
+             geothermal config or a  site-dependent value in the project
+             points CSV). In this case, the resource temperature from
+             the input data will be ignored completely, and the
+             temperature at each location will be determined solely from
+             this input.
+
+        - ``resource_depth`` : Depth to geothermal resource (in m).
+        - ``analysis_type`` : Integer flag representing the plant
+          configuration. If the ``nameplate`` input is to be used to
+          specify the plant capacity, then this flag should be set to 0
+          (this is the default ``reV`` assumption). Otherwise, if the
+          ``num_wells`` input is to be used to specify the plant site,
+          then this flag should be set to 1. Only values of 0 or 1
+          allowed.
+
+        - ``nameplate`` : Geothermal plant size (in kW). Only affects
+          the output if ``analysis_type=0``.
+
+          .. Important:: Unlike wind or solar, ``reV`` geothermal
+             dynamically sets the size of a geothermal plant. In
+             particular, the plant capacity is set to match the resource
+             potential (obtained from the input data) for each site. For
+             this to work, users **must** leave out the ``nameplate``
+             key from the SAM config.
+
+             Alternatively, users *may* specify their own ``nameplate``
+             capacity value (either a single value for all sites in the
+             SAM geothermal config or a site-dependent value in the
+             project points CSV). In this case, the resource potential
+             from the input data will be ignored completely, and the
+             capacity at each location will be determined solely from
+             this input.
+
+        - ``num_wells`` : Number of wells at each plant. This value is
+          used to determined plant capacity if ``analysis_type=1``.
+          Otherwise this input has no effect.
+        - ``num_wells_getem`` : Number of wells assumed at each plant
+          for power block calculations. Only affects power block outputs
+          if ``analysis_type=0`` (otherwise the ``num_wells`` input is
+          used in power block calculations).
+
+          .. Note:: ``reV`` does not currently adjust this value based
+             on the resource input (as it probably should). If any
+             power block outputs are required in the future, there may
+             need to be extra development to set this value based on
+             the dynamically calculated plant size.
+
+        - ``conversion_type`` : Integer flag representing the conversion
+          plant type. Either Binary (0) or Flash (1). Only values of 0
+          or 1 allowed.
+        - ``design_temp`` : EGS plant design temperature (in C). Only
+          affects EGS runs. If this value is set lower than the
+          resource temperature input, ``reV`` will adjust it to match
+          the latter in order to avoid SAM errors.
+        - ``geotherm.cost.inj_prod_well_ratio`` : Fraction representing
+          the injection to production well ratio (0-1). SAM GUI defaults
+          to 0.5 for this value, but it is recommended to set this to
+          the GETEM default of 0.75.
+
+
+    You may also include the following ``reV``-specific keys:
+
+        - ``num_confirmation_wells`` : Number of confirmation wells that
+          can also be used as production wells. This number is used to
+          determined to total number of wells required at each plant,
+          and therefore the total drilling costs. This value defaults to
+          2 (to match the SAM GUI as of 8/1/2023). However, the default
+          value can lead to negative costs if the plant size is small
+          (e.g. only 1 production well is needed, so the costs equal
+          -1 * ``drill_cost_per_well``). This is a limitation of the
+          SAM calculations (as of 8/1/2023), and it is therefore useful
+          to set ``num_confirmation_wells=0`` when performing ``reV``
+          runs for small plant sizes.
+        - ``capital_cost_per_kw`` : Capital cost values in $/kW. If
+          this value is specified in the config, reV calculates and
+          overrides the total ``capital_cost`` value based on the
+          geothermal plant size (capacity) at each location.
+        - ``fixed_operating_cost`` : Fixed operating cost values in
+          $/kW. If this value is specified in the config, reV calculates
+          and overrides the total ``fixed_operating_cost`` value based
+          on the geothermal plant size (capacity) at each location.
+        - ``drill_cost_per_well`` : Drilling cost per well, in $. If
+          this value is specified in the config, reV calculates the
+          total drilling costs based on the number of wells that need to
+          be drilled at each location. The drilling costs are added to
+          the total ``capital_cost`` at each location.
+        - ``reV_outages`` : Specification for ``reV``-scheduled
+          stochastic outage losses. For example::
+
+                outage_info = [
+                    {
+                        'count': 6,
+                        'duration': 24,
+                        'percentage_of_capacity_lost': 100,
+                        'allowed_months': ['January', 'March'],
+                        'allow_outage_overlap': True
+                    },
+                    {
+                        'count': 10,
+                        'duration': 1,
+                        'percentage_of_capacity_lost': 10,
+                        'allowed_months': ['January'],
+                        'allow_outage_overlap': False
+                    },
+                    ...
+                ]
+
+          See the description of
+          :meth:`~reV.losses.scheduled.ScheduledLossesMixin.add_scheduled_losses`
+          or the
+          `reV losses demo notebook <https://tinyurl.com/4d7uutt3/>`_
+          for detailed instructions on how to specify this input.
+        - ``reV_outages_seed`` : Integer value used to seed the RNG
+          used to compute stochastic outage losses.
+        - ``time_index_step`` : Integer representing the step size
+          used to sample the ``time_index`` in the resource data.
+          This can be used to reduce temporal resolution (i.e. for
+          30 minute NSRDB input data, ``time_index_step=1`` yields
+          the full 30 minute time series as output, while
+          ``time_index_step=2`` yields hourly output, and so forth).
+
     """
 
     MODULE = 'geothermal'
     PYSAM = PySamGeothermal
     PYSAM_WEATHER_TAG = "file_name"
     _RESOURCE_POTENTIAL_MULT = 1.001
+    _DEFAULT_NUM_CONFIRMATION_WELLS = 2  # SAM GUI default as of 5/26/23
 
     @staticmethod
     def default():
@@ -1254,13 +1520,14 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
             Time series resource data for a single location with a
             pandas DatetimeIndex. There must be columns for all the
             required variables to run the respective SAM simulation.
-        meta : pd.DataFrame | pd.Series
+        meta : pd.Series
             Meta data corresponding to the resource input for the single
             location. Should include values for latitude, longitude,
             elevation, and timezone.
         """
         super().set_resource_data(resource, meta)
         self._set_resource_temperature(resource)
+        self._set_egs_plant_design_temperature()
         self._set_nameplate_to_match_resource_potential(resource)
         self._set_resource_potential_to_match_gross_output()
         self._set_costs()
@@ -1288,9 +1555,32 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
                      .format(val))
         self.sam_sys_inputs["resource_temp"] = val
 
+    def _set_egs_plant_design_temperature(self):
+        """Set the EGS plant temp to match resource, if necessary"""
+        if self.sam_sys_inputs.get("resource_type") != 1:
+            return  # Not EGS run
+
+        egs_plant_design_temp = self.sam_sys_inputs.get("design_temp", 0)
+        resource_temp = self.sam_sys_inputs["resource_temp"]
+        if egs_plant_design_temp > resource_temp:
+            msg = ('EGS plant design temperature ({}C) exceeds resource '
+                   'temperature ({}C). Lowering EGS plant design temperature '
+                   'to match resource temperature'
+                   .format(egs_plant_design_temp, resource_temp))
+            logger.warning(msg)
+            warn(msg)
+            self.sam_sys_inputs["design_temp"] = resource_temp
+
     def _set_nameplate_to_match_resource_potential(self, resource):
         """Set the nameplate capacity to match the resource potential. """
 
+        if "nameplate" in self.sam_sys_inputs:
+            msg = ('Found "nameplate" input in config! Resource potential '
+                   'from input data will be ignored. Nameplate capacity is {}'
+                   .format(self.sam_sys_inputs["nameplate"]))
+            logger.info(msg)
+            return
+
         val = set(resource["potential_MW"].unique())
         if len(val) > 1:
             msg = ('Found multiple values for "potential_MW" for site {}: {}'
@@ -1299,12 +1589,6 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
             raise InputError(msg)
 
         val = val.pop() * 1000
-        if "nameplate" in self.sam_sys_inputs:
-            msg = ('Setting "nameplate" is not allowed! Updating '
-                   'user input of {} to match the resource potential: {}'
-                   .format(self.sam_sys_inputs["nameplate"], val))
-            logger.warning(msg)
-            warn(msg)
 
         logger.debug("Setting the nameplate to {}".format(val))
         self.sam_sys_inputs["nameplate"] = val
@@ -1340,6 +1624,13 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
         logger.debug("Setting the resource potential to {} MW"
                      .format(gross_gen))
         self.sam_sys_inputs["resource_potential"] = gross_gen
+
+        ncw = self.sam_sys_inputs.pop("num_confirmation_wells",
+                                      self._DEFAULT_NUM_CONFIRMATION_WELLS)
+        self.sam_sys_inputs["prod_and_inj_wells_to_drill"] = (
+            getattr(self.pysam.Outputs, "num_wells_getem_output")
+            - ncw
+            + getattr(self.pysam.Outputs, "num_wells_getem_inj"))
         self["ui_calculations_only"] = 0
 
     def _set_costs(self):
@@ -1354,6 +1645,23 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
                          .format(capital_cost))
             self.sam_sys_inputs["capital_cost"] = capital_cost
 
+        dc_per_well = self.sam_sys_inputs.pop("drill_cost_per_well", None)
+        num_wells = self.sam_sys_inputs.pop("prod_and_inj_wells_to_drill",
+                                            None)
+        if dc_per_well is not None:
+            if num_wells is None:
+                msg = ('Could not determine number of wells to be drilled. '
+                       'No drilling costs added!')
+                logger.warning(msg)
+                warn(msg)
+            else:
+                capital_cost = self.sam_sys_inputs["capital_cost"]
+                drill_cost = dc_per_well * num_wells
+                logger.debug("Setting the drilling cost to ${:,.2f} "
+                             "({:.2f} wells at ${:,.2f} per well)"
+                             .format(drill_cost, num_wells, dc_per_well))
+                self.sam_sys_inputs["capital_cost"] = capital_cost + drill_cost
+
         foc_per_kw = self.sam_sys_inputs.pop("fixed_operating_cost_per_kw",
                                              None)
         if foc_per_kw is not None:
@@ -1375,7 +1683,7 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
             Time series resource data for a single location with a
             pandas DatetimeIndex. There must be columns for all the
             required variables to run the respective SAM simulation.
-        meta : pd.DataFrame | pd.Series
+        meta : pd.Series
             Meta data corresponding to the resource input for the single
             location. Should include values for latitude, longitude,
             and timezone.
@@ -1430,12 +1738,93 @@ class Geothermal(AbstractSamGenerationFromWeatherFile):
 
 
 class AbstractSamWind(AbstractSamGeneration, PowerCurveLossesMixin, ABC):
-    """Base Class for Wind generation from SAM"""
+    """AbstractSamWind"""
 
     def __init__(self, *args, **kwargs):
-        """
-        See docstring for :class:`AbstractSamGeneration` for full input
-        parameter descriptions.
+        """Wind generation from SAM.
+
+        See the PySAM :py:class:`~PySAM.Windpower.Windpower`
+        documentation for the configuration keys required in the
+        `sam_sys_inputs` config. You may also include the following
+        ``reV``-specific keys:
+
+            - ``reV_power_curve_losses`` : A dictionary that can be used
+              to initialize
+              :class:`~reV.losses.power_curve.PowerCurveLossesInput`.
+              For example::
+
+                    reV_power_curve_losses = {
+                        'target_losses_percent': 9.8,
+                        'transformation': 'exponential_stretching'
+                    }
+
+              See the description of the class mentioned above or the
+              `reV losses demo notebook <https://tinyurl.com/4d7uutt3/>`_
+              for detailed instructions on how to specify this input.
+            - ``reV_outages`` : Specification for ``reV``-scheduled
+              stochastic outage losses. For example::
+
+                    outage_info = [
+                        {
+                            'count': 6,
+                            'duration': 24,
+                            'percentage_of_capacity_lost': 100,
+                            'allowed_months': ['January', 'March'],
+                            'allow_outage_overlap': True
+                        },
+                        {
+                            'count': 10,
+                            'duration': 1,
+                            'percentage_of_capacity_lost': 10,
+                            'allowed_months': ['January'],
+                            'allow_outage_overlap': False
+                        },
+                        ...
+                    ]
+
+              See the description of
+              :meth:`~reV.losses.scheduled.ScheduledLossesMixin.add_scheduled_losses`
+              or the
+              `reV losses demo notebook <https://tinyurl.com/4d7uutt3/>`_
+              for detailed instructions on how to specify this input.
+            - ``reV_outages_seed`` : Integer value used to seed the RNG
+              used to compute stochastic outage losses.
+            - ``time_index_step`` : Integer representing the step size
+              used to sample the ``time_index`` in the resource data.
+              This can be used to reduce temporal resolution (i.e. for
+              30 minute input data, ``time_index_step=1`` yields the
+              full 30 minute time series as output, while
+              ``time_index_step=2`` yields hourly output, and so forth).
+
+              .. Note:: The reduced data shape (i.e. after applying a
+                        step size of `time_index_step`) must still be
+                        an integer multiple of 8760, or the execution
+                        will fail.
+
+        Parameters
+        ----------
+        resource : pd.DataFrame
+            Timeseries solar or wind resource data for a single location with a
+            pandas DatetimeIndex.  There must be columns for all the required
+            variables to run the respective SAM simulation. Remapping will be
+            done to convert typical NSRDB/WTK names into SAM names (e.g. DNI ->
+            dn and wind_speed -> windspeed)
+        meta : pd.DataFrame | pd.Series
+            Meta data corresponding to the resource input for the single
+            location. Should include values for latitude, longitude, elevation,
+            and timezone.
+        sam_sys_inputs : dict
+            Site-agnostic SAM system model inputs arguments.
+        site_sys_inputs : dict
+            Optional set of site-specific SAM system inputs to complement the
+            site-agnostic inputs.
+        output_request : list
+            Requested SAM outputs (e.g., 'cf_mean', 'annual_energy',
+            'cf_profile', 'gen_profile', 'energy_yield', 'ppa_price',
+            'lcoe_fcr').
+        drop_leap : bool
+            Drops February 29th from the resource data. If False, December
+            31st is dropped from leap years.
         """
         super().__init__(*args, **kwargs)
         self.add_power_curve_losses()
@@ -1458,7 +1847,7 @@ class WindPower(AbstractSamWind):
             variables to run the respective SAM simulation. Remapping will be
             done to convert typical NSRDB/WTK names into SAM names (e.g. DNI ->
             dn and wind_speed -> windspeed)
-        meta : pd.DataFrame | pd.Series
+        meta : pd.Series
             Meta data corresponding to the resource input for the single
             location. Should include values for latitude, longitude, elevation,
             and timezone.
@@ -1491,18 +1880,17 @@ class WindPower(AbstractSamWind):
 
         if 'rh' in resource:
             # set relative humidity for icing.
-            rh = np.roll(self.ensure_res_len(resource['rh'].values,
-                                             time_index),
-                         int(meta['timezone'] * self.time_interval),
-                         axis=0)
+            rh = self.ensure_res_len(resource['rh'].values, time_index)
+            n_roll = int(meta['timezone'] * self.time_interval)
+            rh = np.roll(rh, n_roll, axis=0)
             data_dict['rh'] = rh.tolist()
 
         # must be set as matrix in [temperature, pres, speed, direction] order
         # ensure that resource array length is multiple of 8760
         # roll the truncated resource array to local timezone
-        temp = np.roll(self.ensure_res_len(resource[var_list].values,
-                                           time_index),
-                       int(meta['timezone'] * self.time_interval), axis=0)
+        temp = self.ensure_res_len(resource[var_list].values, time_index)
+        n_roll = int(meta['timezone'] * self.time_interval)
+        temp = np.roll(temp, n_roll, axis=0)
         data_dict['data'] = temp.tolist()
 
         data_dict['lat'] = meta['latitude']
@@ -1639,7 +2027,7 @@ class MhkWave(AbstractSamGeneration):
             Timeseries resource data for a single location with a
             pandas DatetimeIndex. There must be columns for all the required
             variables to run the respective SAM simulation.
-        meta : pd.DataFrame | pd.Series
+        meta : pd.Series
             Meta data corresponding to the resource input for the single
             location. Should include values for latitude, longitude, elevation,
             and timezone.
@@ -1669,9 +2057,9 @@ class MhkWave(AbstractSamGeneration):
         # ensure that resource array length is multiple of 8760
         # roll the truncated resource array to local timezone
         for var in ['significant_wave_height', 'energy_period']:
-            data_dict[var] = np.roll(
-                self.ensure_res_len(resource[var].values, time_index),
-                int(meta['timezone'] * self.time_interval), axis=0).tolist()
+            arr = self.ensure_res_len(resource[var].values, time_index)
+            n_roll = int(meta['timezone'] * self.time_interval)
+            data_dict[var] = np.roll(arr, n_roll, axis=0).tolist()
 
         data_dict['lat'] = meta['latitude']
         data_dict['lon'] = meta['longitude']
