@@ -21,11 +21,11 @@ class PowerCurve:
 
     Attributes
     ----------
-    wind_speed : :obj:`np.array`
+    wind_speed : :obj:`numpy.array`
         An array containing the wind speeds corresponding to the values
-        in the :attr:`power_curve` array.
-    generation : :obj:`np.array`
-        An array containing the generated power at the corresponding
+        in the :attr:`generation` array.
+    generation : :obj:`numpy.array`
+        An array containing the generated power in kW at the corresponding
         wind speed in the :attr:`wind_speed` array. This input must have
         at least one positive value, and if a cutoff speed is detected
         (see `Warnings` section below), then all values above that wind
@@ -48,12 +48,12 @@ class PowerCurve:
         """
         Parameters
         ----------
-        wind_speed : iter
+        wind_speed : array_like
             An iterable containing the wind speeds corresponding to the
             generated power values in ``generation`` input. The input
             values should all be non-zero.
-        generation : iter
-            An iterable containing the generated power at the
+        generation : array_like
+            An iterable containing the generated power in kW at the
             corresponding wind speed in the ``wind_speed`` input. This
             input must have at least one positive value, and if a cutoff
             speed is detected (see `Warnings` section below), then all
@@ -63,6 +63,7 @@ class PowerCurve:
         self.generation = np.array(generation)
         self._cutoff_wind_speed = None
         self._cutin_wind_speed = None
+        self.i_cutoff = None
 
         _validate_arrays_not_empty(self,
                                    array_names=['wind_speed', 'generation'])
@@ -123,11 +124,24 @@ class PowerCurve:
             ind = np.argmax(self.generation[::-1])
             # pylint: disable=chained-comparison
             if ind > 0 and self.generation[-ind] <= 0:
+                self.i_cutoff = len(self.generation) - ind
                 self._cutoff_wind_speed = self.wind_speed[-ind]
             else:
                 self._cutoff_wind_speed = np.inf
         return self._cutoff_wind_speed
 
+    @property
+    def rated_power(self):
+        """Get the rated power (max power) of the turbine power curve. The
+        units are dependent on the input power curve but this is typically in
+        units of kW.
+
+        Returns
+        -------
+        float
+        """
+        return np.max(self.generation)
+
     def __eq__(self, other):
         return np.isclose(self.generation, other).all()
 
@@ -157,13 +171,13 @@ class PowerCurve:
 
         Parameters
         ----------
-        wind_speed : int | float | list | np.array
+        wind_speed : int | float | list | array_like
             Wind speed value corresponding to the desired power curve
             value.
 
         Returns
         -------
-        float | :obj:`np.array`
+        float | :obj:`numpy.array`
             The power curve value(s) for the input wind speed(s).
         """
         if isinstance(wind_speed, (int, float)):
@@ -203,9 +217,8 @@ class PowerCurveLosses:
     Attributes
     ----------
     power_curve : :obj:`PowerCurve`
-        A :obj:`PowerCurve` object representing the "original
-        powerCurve".
-    wind_resource : :obj:`np.array`
+        The original Power Curve.
+    wind_resource : :obj:`numpy.array`
         An array containing the wind speeds (i.e. wind speed
         distribution) for the site at which the power curve will be
         used. This distribution is used to calculate the annual
@@ -213,7 +226,7 @@ class PowerCurveLosses:
         calculated power curves. The generation values are then compared
         in order to calculate the loss resulting from a transformed
         power curve.
-    weights :obj:`np.array`
+    weights : :obj:`numpy.array`
         An array of the same length as ``wind_resource`` containing
         weights to apply to each generation value calculated for the
         corresponding wind speed.
@@ -223,11 +236,9 @@ class PowerCurveLosses:
         """
         Parameters
         ----------
-        power_curve : PowerCurve
-            A :obj:`PowerCurve` object representing the turbine
-            power curve. This input is treated as the
-            "original" power curve.
-        wind_resource : iter
+        power_curve : :obj:`PowerCurve`
+            The "original" power curve to be adjusted.
+        wind_resource : array_like
             An iterable containing the wind speeds measured at the site
             where this power curve will be applied to calculate
             generation. These values are used to calculate the loss
@@ -236,12 +247,13 @@ class PowerCurveLosses:
             values should all be non-zero, and the units of
             should match the units of the ``power_curve`` input
             (typically, m/s).
-        weights : iter, optional
+        weights : array_like, optional
             An iterable of the same length as ``wind_resource``
             containing weights to apply to each generation value
             calculated for the corresponding wind speed.
-        site : int
-            Site number (gid) for debugging and logging
+        site : int | str, optional
+            Site number (gid) for debugging and logging.
+            By default, ``None``.
         """
 
         self.power_curve = power_curve
@@ -251,7 +263,7 @@ class PowerCurveLosses:
         else:
             self.weights = np.array(weights)
         self._power_gen = None
-        self.site = site
+        self.site = "[unknown]" if site is None else site
 
         _validate_arrays_not_empty(self,
                                    array_names=['wind_resource', 'weights'])
@@ -292,11 +304,10 @@ class PowerCurveLosses:
 
         Parameters
         ----------
-        transformed_power_curve : PowerCurve
-            A :obj:`PowerCurve` object representing the transformed
-            power curve. The power generated with this power curve will
-            be compared with the power generated by the "original" power
-            curve to calculate annual losses.
+        transformed_power_curve : :obj:`PowerCurve`
+            A transformed power curve. The power generated with this
+            power curve will be compared with the power generated by the
+            "original" power curve to calculate annual losses.
 
         Returns
         -------
@@ -331,12 +342,12 @@ class PowerCurveLosses:
         target : float
             Target value for annual generation losses (%).
         transformation : PowerCurveTransformation
-            A :class:`PowerCurveTransformation` class representing the
-            power curve transformation to use.
+            A PowerCurveTransformation class representing the power
+            curve transformation to use.
 
         Returns
         -------
-        :obj:`np.array`
+        :obj:`numpy.array`
             An array containing a transformed power curve that most
             closely yields the ``target`` annual generation losses.
 
@@ -430,18 +441,24 @@ class PowerCurveLossesInput:
         specs : dict
             A dictionary containing specifications for the power curve
             losses. This dictionary must contain the following keys:
-                - `target_losses_percent`
-                    An integer or float value representing the
-                    total percentage of annual energy production that
-                    should be lost due to the power curve
-                    transformation. This value must be in the range
-                    [0, 100].
+
+                - ``target_losses_percent``
+                   An integer or float value representing the
+                   total percentage of annual energy production that
+                   should be lost due to the power curve transformation.
+                   This value must be in the range [0, 100].
+
             The input dictionary can also provide the following optional
             keys:
-                - `transformation` - by default, ``horizontal_translation``
-                    A string representing the type of transformation to
-                    apply to the power curve. This sting must be one of
-                    the keys of :obj:`TRANSFORMATIONS`.
+
+                - ``transformation`` - by default, ``horizontal_translation``
+                  A string representing the type of transformation to
+                  apply to the power curve. This sting must be one of
+                  the keys of :obj:`TRANSFORMATIONS`. See the relevant
+                  transformation class documentation for detailed
+                  information on that type of power curve
+                  transformation.
+
 
         """
         self._specs = specs
@@ -499,18 +516,171 @@ class PowerCurveLossesInput:
 
     @property
     def transformation(self):
-        """PowerCurveTransformation: Requested power curve transformation."""
+        """PowerCurveTransformation: Power curve transformation."""
         return TRANSFORMATIONS[self._transformation_name]
 
 
+class PowerCurveWindResource:
+    """Wind resource data for calculating power curve shift."""
+
+    def __init__(self, temperature, pressure, wind_speed):
+        """Power Curve Wind Resource.
+
+        Parameters
+        ----------
+        temperature : array_like
+            An iterable representing the temperatures at a single site
+            (in C). Must be the same length as the `pressure` and
+            `wind_speed` inputs.
+        pressure : array_like
+            An iterable representing the pressures at a single site
+            (in PA or ATM). Must be the same length as the `temperature`
+            and `wind_speed` inputs.
+        wind_speed : array_like
+            An iterable representing the wind speeds at a single site
+            (in m/s). Must be the same length as the `temperature` and
+            `pressure` inputs.
+        """
+        self._temperatures = np.array(temperature)
+        self._pressures = np.array(pressure)
+        self._wind_speeds = np.array(wind_speed)
+        self.wind_speed_weights = None
+
+    def wind_resource_for_site(self):
+        """Extract scaled wind speeds at the resource site.
+
+        Get the wind speeds for this site, accounting for the scaling
+        done in SAM [1]_ based on air pressure [2]_. These wind speeds
+        can then be used to sample the power curve and obtain generation
+        values.
+
+        Returns
+        -------
+        array-like
+            Array of scaled wind speeds.
+
+        References
+        ----------
+        .. [1] Scaling done in SAM: https://tinyurl.com/2uzjawpe
+        .. [2] SAM Wind Power Reference Manual for explanations on
+           generation and air density calculations (pp. 18):
+           https://tinyurl.com/2p8fjba6
+
+        """
+        if self._pressures.max() < 2:  # units are ATM
+            pressures_pascal = self._pressures * 101325.027383
+        elif self._pressures.min() > 1e4:  # units are PA
+            pressures_pascal = self._pressures
+        else:
+            msg = ("Unable to determine pressure units: pressure values "
+                   "found in the range {:.2f} to {:.2f}. Please make "
+                   "sure input pressures are in units of PA or ATM"
+                   .format(self._pressures.min(), self._pressures.max()))
+            logger.error(msg)
+            raise reVLossesValueError(msg)
+
+        temperatures_K = self._temperatures + 273.15  # originally in celsius
+        specific_gas_constant_dry_air = 287.058  # units: J / kg / K
+        sea_level_air_density = 1.225  # units: kg/m**3 at 15 degrees celsius
+
+        site_air_densities = pressures_pascal / (specific_gas_constant_dry_air
+                                                 * temperatures_K)
+        weights = (sea_level_air_density / site_air_densities) ** (1 / 3)
+        return self._wind_speeds / weights
+
+    @property
+    def wind_speeds(self):
+        """:obj:`numpy.array`: Array of adjusted wind speeds. """
+        return self.wind_resource_for_site()
+
+
+class _PowerCurveWindDistribution:
+    """`PowerCurveWindResource` interface mocker for wind distributions. """
+
+    def __init__(self, speeds, weights):
+        """Power Curve Wind Resource for Wind Distributions.
+
+        Parameters
+        ----------
+        speeds : array_like
+            An iterable representing the wind speeds at a single site
+            (in m/s). Must be the same length as the `weights` input.
+        weights : array_like
+            An iterable representing the wind speed weights at a single
+            site. Must be the same length as the `speeds` input.
+        """
+        self.wind_speeds = np.array(speeds)
+        self.wind_speed_weights = np.array(weights)
+
+
+def adjust_power_curve(power_curve, resource_data, target_losses, site=None):
+    """Adjust power curve to account for losses.
+
+    This function computes a new power curve that accounts for the
+    loss percentage specified from the target loss.
+
+    Parameters
+    ----------
+    power_curve : :obj:`PowerCurve`
+        Power curve to be adjusted to match target losses.
+    resource_data : :obj:`PowerCurveWindResource`
+        Resource data for the site being investigated.
+    target_losses : :obj:`PowerCurveLossesInput`
+        Target loss and power curve shift info.
+    site : int | str, optional
+        Site number (gid) for debugging and logging.
+        By default, ``None``.
+
+    Returns
+    -------
+    :obj:`PowerCurve`
+        Power Curve shifted to meet the target losses. Power Curve is
+        not adjusted if all wind speeds are above the cutout or below
+        the cutin speed.
+
+    See Also
+    --------
+    :obj:`PowerCurveLosses` : Power curve re-calculation.
+    """
+    site = "[unknown]" if site is None else site
+
+    if (resource_data.wind_speeds <= power_curve.cutin_wind_speed).all():
+        msg = ("All wind speeds for site {} are below the wind speed "
+               "cutin ({} m/s). No power curve adjustments made!"
+               .format(site, power_curve.cutin_wind_speed))
+        logger.warning(msg)
+        warnings.warn(msg, reVLossesWarning)
+        return power_curve
+
+    if (resource_data.wind_speeds >= power_curve.cutoff_wind_speed).all():
+        msg = ("All wind speeds for site {} are above the wind speed "
+               "cutoff ({} m/s). No power curve adjustments made!"
+               .format(site, power_curve.cutoff_wind_speed))
+        logger.warning(msg)
+        warnings.warn(msg, reVLossesWarning)
+        return power_curve
+
+    pc_losses = PowerCurveLosses(power_curve, resource_data.wind_speeds,
+                                 resource_data.wind_speed_weights, site=site)
+
+    logger.debug("Transforming power curve using the {} transformation to "
+                 "meet {}% loss target..."
+                 .format(target_losses.transformation, target_losses.target))
+
+    new_curve = pc_losses.fit(target_losses.target,
+                              target_losses.transformation)
+    logger.debug("Transformed power curve: {}".format(list(new_curve)))
+    return new_curve
+
+
 class PowerCurveLossesMixin:
     """Mixin class for :class:`reV.SAM.generation.AbstractSamWind`.
 
-    Warning
-    -------
+    Warnings
+    --------
     Using this class for anything except as a mixin for
-    :class:`~reV.SAM.generation.AbstractSamWind` may result in unexpected
-    results and/or errors.
+    :class:`~reV.SAM.generation.AbstractSamWind` may result in
+    unexpected results and/or errors.
     """
 
     POWER_CURVE_CONFIG_KEY = 'reV_power_curve_losses'
@@ -528,43 +698,16 @@ class PowerCurveLossesMixin:
 
         See Also
         --------
-        :class:`PowerCurveLosses` : Power curve re-calculation.
-
+        :func:`adjust_power_curve` : Power curve shift calculation.
         """
         loss_input = self._user_power_curve_input()
         if not loss_input:
             return
 
-        wind_resource, weights = self.wind_resource_from_input()
-        power_curve = self.input_power_curve
-
-        if (wind_resource <= power_curve.cutin_wind_speed).all():
-            msg = ("All wind speeds for site {} are below the wind speed "
-                   "cutin ({} m/s). No power curve adjustments made!"
-                   .format(getattr(self, "_site", "[unknown]"),
-                           power_curve.cutin_wind_speed))
-            logger.warning(msg)
-            warnings.warn(msg, reVLossesWarning)
-            return
-
-        if (wind_resource >= power_curve.cutoff_wind_speed).all():
-            msg = ("All wind speeds for site {} are above the wind speed "
-                   "cutoff ({} m/s). No power curve adjustments made!"
-                   .format(getattr(self, "_site", "[unknown]"),
-                           power_curve.cutoff_wind_speed))
-            logger.warning(msg)
-            warnings.warn(msg, reVLossesWarning)
-            return
-
-        pc_losses = PowerCurveLosses(power_curve, wind_resource, weights,
-                                     site=getattr(self, 'site', None))
-
-        logger.debug("Transforming power curve using the {} transformation to "
-                     "meet {}% loss target..."
-                     .format(loss_input.transformation, loss_input.target))
-
-        new_curve = pc_losses.fit(loss_input.target, loss_input.transformation)
-        logger.debug("Transformed power curve: {}".format(list(new_curve)))
+        resource = self.wind_resource_from_input()
+        site = getattr(self, 'site', "[unknown]")
+        new_curve = adjust_power_curve(self.input_power_curve, resource,
+                                       loss_input, site=site)
         self.sam_sys_inputs['wind_turbine_powercurve_powerout'] = new_curve
 
     def _user_power_curve_input(self):
@@ -589,7 +732,7 @@ class PowerCurveLossesMixin:
 
     @property
     def input_power_curve(self):
-        """PowerCurve: Original power curve for site. """
+        """:obj:`PowerCurve`: Original power curve for site. """
         wind_speed = self.sam_sys_inputs['wind_turbine_powercurve_windspeeds']
         generation = self.sam_sys_inputs['wind_turbine_powercurve_powerout']
         return PowerCurve(wind_speed, generation)
@@ -599,10 +742,8 @@ class PowerCurveLossesMixin:
 
         Returns
         -------
-        wind_speeds : array-like
-            Array of wind speeds.
-        weights : array-like | ``None``
-            Array of weights for corresponding wind speeds, or ``None``.
+        :obj:`PowerCurveWindResource`
+            Wind resource used to compute power curve shift.
 
         Raises
         ------
@@ -611,9 +752,13 @@ class PowerCurveLossesMixin:
             'wind_resource_model_choice'.
         """
         if self['wind_resource_model_choice'] == 0:
-            return self.wind_resource_for_site(), None
+            temperatures, pressures, wind_speeds, __, = map(
+                np.array, zip(*self['wind_resource_data']['data'])
+            )
+            return PowerCurveWindResource(temperatures, pressures, wind_speeds)
         elif self['wind_resource_model_choice'] == 2:
-            return self.wind_resource_from_distribution()
+            wrd = np.array(self['wind_resource_distribution'])
+            return _PowerCurveWindDistribution(wrd[:, 0], wrd[:, -1])
         else:
             msg = ("reV power curve losses cannot be used with "
                    "'wind_resource_model_choice' = {}"
@@ -621,59 +766,6 @@ class PowerCurveLossesMixin:
             logger.error(msg)
             raise reVLossesValueError(msg)
 
-    def wind_resource_for_site(self):
-        """Extract scaled wind speeds at the site.
-
-        Get the wind resource (wind speeds) for this site, accounting
-        for the scaling done in SAM based on air pressure. These wind
-        speeds can then be used to sample the power curve and obtain
-        generation values.
-
-        Returns
-        -------
-        array-like
-            Array of scaled wind speeds.
-
-        References
-        ----------
-        Scaling done in SAM:
-            https://tinyurl.com/2uzjawpe
-        SAM Wind Power Reference Manual for explanations on generation
-        and air density calculations (pp. 18):
-            https://tinyurl.com/2p8fjba6
-
-        """
-
-        temperatures, pressures, wind_speeds, __, = map(
-            np.array, zip(*self['wind_resource_data']['data'])
-        )
-
-        pressures_pascal = pressures * 101325.0273830  # originally in atm
-        temperatures_K = temperatures + 273.15  # originally in celsius
-        specific_gas_constant_dry_air = 287.058  # units: J / kg / K
-        sea_level_air_density = 1.225  # units: kg/m**3 at 15 degrees celsius
-
-        site_air_densities = pressures_pascal / (specific_gas_constant_dry_air
-                                                 * temperatures_K)
-        weights = (sea_level_air_density / site_air_densities) ** (1 / 3)
-        return wind_speeds / weights
-
-    def wind_resource_from_distribution(self):
-        """Extract wind speeds and weights from resource distribution.
-
-        Returns
-        -------
-        wind_speeds : array-like
-            Array of wind speeds.
-        weights : array-like
-            Array of weights for corresponding wind speeds.
-        """
-
-        wrd = np.array(self['wind_resource_distribution'])
-        wind_speeds = wrd[:, 0]
-        weights = wrd[:, -1]
-        return wind_speeds, weights
-
 
 class AbstractPowerCurveTransformation(ABC):
     """Abstract base class for power curve transformations.
@@ -693,24 +785,23 @@ class AbstractPowerCurveTransformation(ABC):
     Attributes
     ----------
     power_curve : :obj:`PowerCurve`
-        A :obj:`PowerCurve` object representing the "original" power
-        curve.
+        The "original" input power curve.
     """
 
     def __init__(self, power_curve):
-        """
+        """Abstract Power Curve Transformation class.
+
         Parameters
         ----------
-        power_curve : PowerCurve
-            A :obj:`PowerCurve` object representing the turbine
-            power curve. This input is treated as the "original" power
-            curve.
+        power_curve : :obj:`PowerCurve`
+            The turbine power curve. This input is treated as the
+            "original" power curve.
         """
         self.power_curve = power_curve
         self._transformed_generation = None
 
-    def _validate_shifted_power_curve(self, new_curve):
-        """Ensure new power curve has some non-zero generation. """
+    def _validate_non_zero_generation(self, new_curve):
+        """Ensure new power curve has some non-zero generation."""
         mask = (self.power_curve.wind_speed
                 <= self.power_curve.cutoff_wind_speed)
         min_expected_power_gen = self.power_curve[self.power_curve > 0].min()
@@ -723,6 +814,18 @@ class AbstractPowerCurveTransformation(ABC):
             logger.error(msg)
             raise reVLossesValueError(msg)
 
+    def _validate_same_cutoff(self, new_curve):
+        """Validate that the new power curve has the same high-wind cutout as
+        the original curve."""
+        old_cut = self.power_curve.cutoff_wind_speed
+        new_cut = new_curve.cutoff_wind_speed
+        if old_cut != new_cut:
+            msg = ('Original power curve windspeed cutout is {}m/s and new '
+                   'curve cutout is {}m/s. Something went wrong!'
+                   .format(old_cut, new_cut))
+            logger.error(msg)
+            raise reVLossesValueError(msg)
+
     @abstractmethod
     def apply(self, transformation_var):
         """Apply a transformation to the original power curve.
@@ -731,7 +834,7 @@ class AbstractPowerCurveTransformation(ABC):
         ----------
         transformation_var : : float
             A single variable controlling the "strength" of the
-            transformation. The :class:`PowerCurveLosses` object will
+            transformation. The :obj:`PowerCurveLosses` object will
             run an optimization using this variable to fit the target
             annual losses incurred with the transformed power curve
             compared to the original power curve using the given wind
@@ -772,13 +875,23 @@ class AbstractPowerCurveTransformation(ABC):
             logger.error(msg)
             raise NotImplementedError(msg)
 
-        mask = (self.power_curve.wind_speed
-                >= self.power_curve.cutoff_wind_speed)
-        self._transformed_generation[mask] = 0
+        if not np.isinf(self.power_curve.cutoff_wind_speed):
+            mask = (self.power_curve.wind_speed
+                    >= self.power_curve.cutoff_wind_speed)
+            self._transformed_generation[mask] = 0
+            new_curve = PowerCurve(self.power_curve.wind_speed,
+                                   self._transformed_generation)
+            self._validate_non_zero_generation(new_curve)
+
+            i_max = np.argmax(self._transformed_generation)
+            i_cutoff = self.power_curve.i_cutoff
+            rated_power = self.power_curve.rated_power
+            self._transformed_generation[i_max:i_cutoff] = rated_power
 
         new_curve = PowerCurve(self.power_curve.wind_speed,
                                self._transformed_generation)
-        self._validate_shifted_power_curve(new_curve)
+        self._validate_non_zero_generation(new_curve)
+        self._validate_same_cutoff(new_curve)
         return new_curve
 
     @property
@@ -820,8 +933,7 @@ class HorizontalTranslation(AbstractPowerCurveTransformation):
     Attributes
     ----------
     power_curve : :obj:`PowerCurve`
-        A :obj:`PowerCurve` object representing the "original" power
-        curve.
+        The "original" input power curve.
 
     Warnings
     --------
@@ -890,8 +1002,7 @@ class LinearStretching(AbstractPowerCurveTransformation):
     Attributes
     ----------
     power_curve : :obj:`PowerCurve`
-        A :obj:`PowerCurve` object representing the "original" power
-        curve.
+        The "original" input power curve.
     """
 
     def apply(self, transformation_var):
@@ -953,8 +1064,7 @@ class ExponentialStretching(AbstractPowerCurveTransformation):
     Attributes
     ----------
     power_curve : :obj:`PowerCurve`
-        A :obj:`PowerCurve` object representing the "original" power
-        curve.
+        The "original" input power curve.
     """
 
     def apply(self, transformation_var):
