@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# pylint: disable=no-member
 """
 reV-NRWAL analysis module.
 
@@ -27,62 +28,124 @@ logger = logging.getLogger(__name__)
 
 
 class RevNrwal:
-    """Framework to handle reV-NRWAL analysis."""
+    """RevNrwal"""
 
-    # Default columns from the site_data table to join to the output meta data
     DEFAULT_META_COLS = ('config', )
+    """Columns from the `site_data` table to join to the output meta data"""
 
     def __init__(self, gen_fpath, site_data, sam_files, nrwal_configs,
                  output_request, save_raw=True, meta_gid_col='gid',
-                 site_meta_cols=None,
-                 ):
-        """
+                 site_meta_cols=None):
+        """Framework to handle reV-NRWAL analysis.
+
+        ``reV`` NRWAL analysis runs ``reV`` data through the NRWAL
+        compute library. Everything in this module operates on the
+        spatiotemporal resolution of the ``reV`` generation output file
+        (usually the wind or solar resource resolution but could also be
+        the supply curve resolution after representative profiles is
+        run).
+
         Parameters
         ----------
         gen_fpath : str
-            Full filepath to reV generation or rep_profiles h5 output file.
-            Anything in the output_request is added and/or manipulated in this
-            file.
+            Full filepath to HDF5 file with ``reV`` generation or
+            rep_profiles output. Anything in the `output_request` input
+            is added to and/or manipulated within this file.
+
+            .. Note:: If executing ``reV`` from the command line, this
+              input can also be ``"PIPELINE"`` to parse the output of
+              one of the previous step and use it as input to this call.
+              However, note that duplicate executions of ``reV``
+              commands prior to this one within the pipeline may
+              invalidate this parsing, meaning the `gen_fpath` input
+              will have to be specified manually.
+
         site_data : str | pd.DataFrame
-            Site-specific input data for NRWAL calculation. String should be a
-            filepath that points to a csv, DataFrame is pre-extracted data.
-            Rows match sites, columns are input keys. Need a "gid" column that
-            corresponds to the "meta_gid_col" in the gen_fpath meta data and a
-            "config" column that corresponds to the nrwal_configs input. Only
-            sites with a gid in this file's "gid" column will be run through
-            NRWAL.
-        sam_files : dict
-            Dictionary lookup of config_id (keys) mapped to config filepaths
-            (values). The same config_id values will be used from the
-            nrwal_configs lookup input.
+            Site-specific input data for NRWAL calculation.If this input
+            is a string, it should be a path that points to a CSV file.
+            Otherwise, this input should be a DataFrame with
+            pre-extracted site data. Rows in this table should match
+            the `meta_gid_col` in the `gen_fpath` meta data input
+            sites via a ``gid`` column. A ``config`` column must also be
+            provided that corresponds to the `nrwal_configs` input. Only
+            sites with a gid in this file's ``gid`` column will be run
+            through NRWAL.
+        sam_files : dict | str
+            A dictionary mapping SAM input configuration ID(s) to SAM
+            configuration(s). Keys are the SAM config ID(s) which
+            correspond to the keys in the `nrwal_configs` input. Values
+            for each key are either a path to a corresponding SAM
+            config file or a full dictionary of SAM config inputs. For
+            example::
+
+                sam_files = {
+                    "default": "/path/to/default/sam.json",
+                    "onshore": "/path/to/onshore/sam_config.yaml",
+                    "offshore": {
+                        "sam_key_1": "sam_value_1",
+                        "sam_key_2": "sam_value_2",
+                        ...
+                    },
+                    ...
+                }
+
+            This input can also be a string pointing to a single SAM
+            config file. In this case, the ``config`` column of the
+            CSV points input should be set to ``None`` or left out
+            completely. See the documentation for the ``reV`` SAM class
+            (e.g. :class:`reV.SAM.generation.WindPower`,
+            :class:`reV.SAM.generation.PvWattsv8`,
+            :class:`reV.SAM.generation.Geothermal`, etc.) for
+            documentation on the allowed and/or required SAM config file
+            inputs.
         nrwal_configs : dict
-            Dictionary lookup of config_id (keys) mapped to config filepaths
-            (values). The same config_id values will be used from the
-            sam_files lookup in project_points
+            A dictionary mapping SAM input configuration ID(s) to NRWAL
+            configuration(s). Keys are the SAM config ID(s) which
+            correspond to the keys in the `sam_files` input. Values
+            for each key are either a path to a corresponding NRWAL YAML
+            or JSON config file or a full dictionary of NRWAL config
+            inputs. For example::
+
+                nrwal_configs = {
+                    "default": "/path/to/default/nrwal.json",
+                    "onshore": "/path/to/onshore/nrwal_config.yaml",
+                    "offshore": {
+                        "nrwal_key_1": "nrwal_value_1",
+                        "nrwal_key_2": "nrwal_value_2",
+                        ...
+                    },
+                    ...
+                }
+
         output_request : list | tuple
-            List of output dataset names you want written to the gen_fpath
-            file. Any key from the NRWAL configs or any of the inputs
-            (site_data or sam_files) is available to be exported as an output
-            dataset. If you want to manipulate a dset like cf_mean from
-            gen_fpath and include it in the output_request, you should set
-            save_raw=True and then in the NRWAL equations use cf_mean_raw as
-            the input and then define cf_mean as the manipulated data that will
-            be included in the output_request.
-        save_raw : bool
-            Flag to save a copy of existing datasets in gen_fpath that are part
-            of the output_request. For example, if you request cf_mean in
-            output_request and manipulate the cf_mean dataset in the NRWAL
-            equations, the original cf_mean will be archived under the
-            "cf_mean_raw" dataset in gen_fpath.
-        meta_gid_col : str
-            Column label in the source meta data from gen_fpath that contains
-            the unique gid identifier. This will be joined to the site_data
-            "gid" column
-        site_meta_cols : list | tuple | None
-            Column labels from site_data to be added to the meta data table in
-            gen_fpath. None (default) will use class variable
-            DEFAULT_META_COLS, and any additional cols requested here will be
-            added to DEFAULT_META_COLS.
+            List of output dataset names to be written to the
+            `gen_fpath` file. Any key from the NRWAL configs or any of
+            the inputs (site_data or sam_files) is available to be
+            exported as an output dataset. If you want to manipulate a
+            dset like ``cf_mean`` from `gen_fpath` and include it in the
+            `output_request`, you should set ``save_raw=True`` and then
+            use ``cf_mean_raw`` in the NRWAL equations as the input.
+            This allows you to define an equation in the NRWAL configs
+            for a manipulated ``cf_mean`` output that can be included in
+            the `output_request` list.
+        save_raw : bool, optional
+            Flag to save an initial ("raw") copy of input datasets from
+            `gen_fpath` that are also part of the `output_request`. For
+            example, if you request ``cf_mean`` in output_request but
+            also manipulate the ``cf_mean`` dataset in the NRWAL
+            equations, the original ``cf_mean`` will be archived under
+            the ``cf_mean_raw`` dataset in `gen_fpath`.
+            By default, ``True``.
+        meta_gid_col : str, optional
+            Column label in the source meta data from `gen_fpath` that
+            contains the unique gid identifier. This will be joined to
+            the site_data ``gid`` column. By default, ``"gid"``.
+        site_meta_cols : list | tuple, optional
+            Column labels from `site_data` to be added to the meta data
+            table in `gen_fpath`. If ``None``, only the columns in
+            :attr:`DEFAULT_META_COLS` will be added. Any columns
+            requested via this input will be considered *in addition to*
+            the :attr:`DEFAULT_META_COLS`. By default, ``None``.
         """
 
         log_versions(logger)
@@ -98,6 +161,7 @@ class RevNrwal:
         self._meta_out = None
         self._time_index = None
         self._save_raw = save_raw
+        self._nrwal_inputs = self._out = None
 
         self._nrwal_configs = {k: NrwalConfig(v) for k, v in
                                nrwal_configs.items()}
@@ -119,11 +183,6 @@ class RevNrwal:
         self._project_points = pc.project_points
 
         self._sam_sys_inputs = self._parse_sam_sys_inputs()
-        self._preflight_checks()
-        self.save_raw_dsets()
-        self._nrwal_inputs = self._get_input_data()
-        self._out = self._init_outputs()
-
         meta_gids = self.meta_source[self._meta_gid_col].values
         logger.info('Finished initializing NRWAL analysis module for "{}" '
                     '{} through {} with {} total generation points and '
@@ -581,6 +640,11 @@ class RevNrwal:
     def run_nrwal(self):
         """Run analysis via the NRWAL analysis library"""
 
+        self._preflight_checks()
+        self.save_raw_dsets()
+        self._nrwal_inputs = self._get_input_data()
+        self._out = self._init_outputs()
+
         for i, (cid, nrwal_config) in enumerate(self._nrwal_configs.items()):
             output_mask = self._site_data['config'].values == cid
             logger.info('Running NRWAL config {} of {}: "{}" and applying '
@@ -640,7 +704,13 @@ class RevNrwal:
                                     attrs=f.attrs[dset])
 
     def write_to_gen_fpath(self):
-        """Save NRWAL outputs to input generation fpath file."""
+        """Save NRWAL outputs to input generation fpath file.
+
+        Returns
+        -------
+        str
+            Path to output file.
+        """
 
         logger.info('Writing NRWAL outputs to: {}'.format(self._gen_fpath))
         write_all = self.analysis_mask.all()
@@ -682,77 +752,99 @@ class RevNrwal:
 
         logger.info('Finished writing NRWAL outputs to: {}'
                     .format(self._gen_fpath))
+        return self._gen_fpath
 
-    @classmethod
-    def run(cls, gen_fpath, site_data, sam_files, nrwal_configs,
-            output_request, save_raw=True, meta_gid_col='gid',
-            site_meta_cols=None):
-        """Initialize and run the NRWAL analysis object.
+    def write_meta_to_csv(self, out_fpath=None):
+        """Combine NRWAL outputs with meta and write to output csv.
 
         Parameters
         ----------
-        gen_fpath : str
-            Full filepath to reV generation or rep_profiles h5 output file.
-            Anything in the output_request is added and/or manipulated in this
-            file.
-        site_data : str | pd.DataFrame
-            Site-specific input data for NRWAL calculation. String should be a
-            filepath that points to a csv, DataFrame is pre-extracted data.
-            Rows match sites, columns are input keys. Need a "gid" column that
-            corresponds to the "meta_gid_col" in the gen_fpath meta data and a
-            "config" column that corresponds to the nrwal_configs input. Only
-            sites with a gid in this file's "gid" column will be run through
-            NRWAL.
-        sam_files : dict
-            Dictionary lookup of config_id (keys) mapped to config filepaths
-            (values). The same config_id values will be used from the
-            nrwal_configs lookup input.
-        nrwal_configs : dict
-            Dictionary lookup of config_id (keys) mapped to config filepaths
-            (values). The same config_id values will be used from the
-            sam_files lookup in project_points
-        output_request : list | tuple
-            List of output dataset names you want written to the gen_fpath
-            file. Any key from the NRWAL configs or any of the inputs
-            (site_data or sam_files) is available to be exported as an output
-            dataset. If you want to manipulate a dset like cf_mean from
-            gen_fpath and include it in the output_request, you should set
-            save_raw=True and then in the NRWAL equations use cf_mean_raw as
-            the input and then define cf_mean as the manipulated data that will
-            be included in the output_request.
-        save_raw : bool
-            Flag to save a copy of existing datasets in gen_fpath that are part
-            of the output_request. For example, if you request cf_mean in
-            output_request and manipulate the cf_mean dataset in the NRWAL
-            equations, the original cf_mean will be archived under the
-            "cf_mean_raw" dataset in gen_fpath.
-        meta_gid_col : str
-            Column label in the source meta data from gen_fpath that contains
-            the unique gid identifier. This will be joined to the site_data
-            "gid" column
-        site_meta_cols : list | tuple | None
-            Column labels from site_data to be added to the meta data table in
-            gen_fpath. None (default) will use class variable
-            DEFAULT_META_COLS, and any additional cols requested here will be
-            added to DEFAULT_META_COLS.
+        out_fpath : str, optional
+            Full path to output NRWAL CSV file. The file path does not
+            need to include file ending - it will be added automatically
+            if missing. If ``None``, the generation HDF5 filepath will
+            be converted to a CSV out path by replacing the ".h5" file
+            ending with ".csv". By default, ``None``.
 
         Returns
         -------
-        obj : RevNrwal
-            Instantiated and run RevNrwal analysis object.
+        str
+            Path to output file.
         """
+        if out_fpath is None:
+            out_fpath = self._gen_fpath.replace(".h5", ".csv")
+        elif not out_fpath.endswith(".csv"):
+            out_fpath = "{}.csv".format(out_fpath)
 
-        obj = cls(gen_fpath, site_data, sam_files, nrwal_configs,
-                  output_request,
-                  save_raw=save_raw,
-                  meta_gid_col=meta_gid_col,
-                  site_meta_cols=site_meta_cols)
+        logger.info('Writing NRWAL outputs to: {}'.format(out_fpath))
+        meta_out = self.meta_out[self.analysis_mask].copy()
 
-        if any(obj.analysis_gids):
-            obj.run_nrwal()
-            obj.check_outputs()
-            obj.write_to_gen_fpath()
+        for dset, arr in self._out.items():
+            if len(arr.shape) != 1 or arr.shape[0] != meta_out.shape[0]:
+                msg = ('Skipping output {!r}: shape {} cannot be combined '
+                       'with meta of shape {}!'
+                       .format(dset, arr.shape, meta_out.shape))
+                logger.warning(msg)
+                warn(msg)
+                continue
+            meta_out[dset] = arr
+
+        meta_out.to_csv(out_fpath, index=False)
+        logger.info('Finished writing NRWAL outputs to: {}'.format(out_fpath))
+        return out_fpath
+
+    def run(self, csv_output=False, out_fpath=None):
+        """Run NRWAL analysis.
+
+        Parameters
+        ----------
+        csv_output : bool, optional
+            Option to write H5 file meta + all requested outputs to
+            CSV file instead of storing in the HDF5 file directly. This
+            can be useful if the same HDF5 file is used for multiple
+            sets of NRWAL runs. Note that all requested output datasets
+            must be 1-dimensional in order to fir within the CSV output.
+
+            .. Important:: This option is not compatible with
+              ``save_raw=True``. If you set ``csv_output=True``, then
+              the `save_raw` option is forced to be ``False``.
+              Therefore, make sure that you do not have any references
+              to "input_dataset_name_raw" in your NRWAL config. If you
+              need to manipulate an input dataset, save it to a
+              different output name in the NRWAL config or manually add
+              an "input_dataset_name_raw" dataset to your generation
+              HDF5 file before running NRWAL.
+
+            By default, ``False``.
+        out_fpath : str, optional
+            This option has no effect if ``csv_output=False``.
+            Otherwise, this should be the full path to output NRWAL CSV
+            file. The file path does not need to include file ending -
+            it will be added automatically if missing. If ``None``, the
+            generation HDF5 filepath will be converted to a CSV out path
+            by replacing the ".h5" file ending with ".csv".
+            By default, ``None``.
+
+        Returns
+        -------
+        str
+            Path to output file.
+        """
+        if csv_output and self._save_raw:
+            msg = ("`save_raw` option not allowed with `csv_output`. Setting"
+                   "`save_raw=False`")
+            logger.warning(msg)
+            warn(msg)
+            self._save_raw = False
+
+        if any(self.analysis_gids):
+            self.run_nrwal()
+            self.check_outputs()
+            if csv_output:
+                out_fp = self.write_meta_to_csv(out_fpath)
+            else:
+                out_fp = self.write_to_gen_fpath()
 
         logger.info('NRWAL module complete!')
 
-        return obj
+        return out_fp
